@@ -2,7 +2,7 @@
 #' 
 #' @param y A list of \emph{n} vectors containing the observed values for each individual. Missing values specified by \code{NA}s are supported for dense case (\code{dataType='dense'}).
 #' @param t A list of \emph{n} vectors containing the observation time points for each individual corresponding to y.
-#' @param k A scalar defining the number of clusters to define; default 3.
+#' @param k A scalar defining the number of clusters to define; default 3. Values that define very small clusters (eg.cluster size <=3) will potentially err.
 #' @param kSeed A scalar valid seed number to ensure replication; default: 123
 #' @param maxIter A scalar defining the maximum number of iterations allowed; default 20, common for both the simple kmeans initially and the subsequent k-centres
 #' @param optnsSW A list of options control parameters specified by \code{list(name=value)} to be used for sample-wide FPCA; by default: "list( methodMuCovEst ='smooth', FVEthreshold= 0.90, methodBwCov = 'GCV', methodBwMu = 'GCV' )". See `Details in ?FPCA'.
@@ -16,7 +16,7 @@
 #' @examples
 #' data(medfly25) 
 #' Flies <- MakeFPCAInputs(medfly25$ID, medfly25$Days, medfly25$nEggs)
-#' kcfcObj <- kCFC(Flies$Ly[1:250], Flies$Lt[1:250], # using only 250 for speed consideration 
+#' kcfcObj <- kCFC(Flies$Ly[1:150], Flies$Lt[1:150], # using only 150 for speed consideration 
 #'                  optnsSW = list(methodMuCovEst = 'smooth', userBwCov = 2, FVEthreshold = 0.90),
 #'                  optnsCS = list(methodMuCovEst = 'smooth', userBwCov = 2, FVEthreshold = 0.70))
 #' @references
@@ -48,6 +48,10 @@ kCFC = function(y, t, k = 3, kSeed = 123, maxIter = 125,
   initialClustering <- kmeans( fpcaObjY$xiEst, centers = k, algorithm = "MacQueen", iter.max = maxIter)
   clustConf0 <- as.factor(initialClustering$cluster)
   indClustIds <- lapply(levels(clustConf0), function(u) which(clustConf0 == u) )
+  if( any( min( sapply( indClustIds, length)) <= c(3)) ){
+    stop(paste0("kCFC stopped during the initial k-means step. The smallest cluster has three (or less) curves. " ,
+                "Consider using a smaller number of clusters (k) or a different random seed (kSeed)."))
+  }
   listOfFPCAobjs <- lapply(indClustIds, function(u) FPCA(y[u], t[u], optnsCS) )
   
   ## Iterative clustering
@@ -58,12 +62,12 @@ kCFC = function(y, t, k = 3, kSeed = 123, maxIter = 125,
   for(j in seq_len(maxIter)){ 
     
     # Get new costs and relevant cluster configuration
-    iseCosts       <- sapply(listOfFPCAobjs, function(u) GetISEfromFPCA(u, ymat))
+    iseCosts       <- sapply(listOfFPCAobjs, function(u) GetISEfromFPCA(u, y,t,ymat))
     clustConf[[j]] <- as.factor(apply(iseCosts, 1, which.min))
     
     # Check that clustering progressed reasonably 
-    #ie. Still having k clster AND the minimum cluster size is reasonable
-    if( (length(unique(clustConf[[j]])) < k) || min(summary(clustConf[[j]])) < 0.01 * N){   
+    #ie. Still having k clster AND the minimum cluster size is reasonable 
+    if( (length(unique(clustConf[[j]])) < k) || any( min(summary(clustConf[[j]])) <= c(0.01 * N,3))){
       convInfo <- ifelse( length(unique(clustConf[[j]])) < k , "LostCluster", "TinyCluster")
       break;
     }
@@ -95,15 +99,16 @@ kCFC = function(y, t, k = 3, kSeed = 123, maxIter = 125,
   return( kCFCobj )
 }  
 
-GetISEfromFPCA = function(fpcaObj,ymat){
+GetISEfromFPCA = function(fpcaObj,y,t,ymat){
   # First get the fitted curves for all the sample based on the mu/phi/lambda/sigma2
   # of 'fpcaObj' and then calculate their associated ISE; 'iseCost' is a n-dim vector.
   phiObs <- ConvertSupport(fpcaObj$workGrid, fpcaObj$obsGrid, phi=fpcaObj$phi)
   muObs  <- ConvertSupport(fpcaObj$workGrid, fpcaObj$obsGrid, mu=fpcaObj$mu)
   
-  numIntResults <- GetINScores(ymat = ymat, t = fpcaObj$obsGrid, optns = fpcaObj$optns, mu = muObs, 
-                               lambda = fpcaObj$lambda, phi = phiObs, sigma2 = fpcaObj$sigma2) 
+  numIntResults <- mapply(function(yvec,tvec)
+    GetINScores(yvec, tvec,optns= fpcaObj$optns,fpcaObj$obsGrid,mu = muObs,lambda =fpcaObj$lambda ,phi = phiObs,sigma2 = fpcaObj$sigma2),y,t)
   
-  iseCost <- apply((numIntResults[['fittedY']] - ymat)^2, 1, function(y) {notNA <- !is.na(y);  trapzRcpp(X = fpcaObj$obsGrid[notNA], Y = y[notNA])}) 
+  fittedYmat = List2Mat(numIntResults[3,],t)
+  iseCost <- apply((fittedYmat - ymat)^2, 1, function(y) {notNA <- !is.na(y);  trapzRcpp(X = fpcaObj$obsGrid[notNA], Y = y[notNA])}) 
   return( iseCost )
 }
