@@ -18,12 +18,12 @@
 #' @export
 print.FPCA <- function(x, ...){
   obj = x;
-  message(paste0("Functional Principal Components Object for", tolower(obj$optns$dataType), "data.\n\n"))
-  message(paste0("The optimal number of components selected is:", length(obj$lambda),"and \nthe first K (<=3) eigenvalues are: "))
+  cat(paste0("Functional Principal Components Object for ", tolower(obj$optns$dataType), " data.\n\n"))
+  cat(paste0("The optimal number of components selected is: ", length(obj$lambda), ", and \nthe first K (<=3) eigenvalues are: "))
   if ( length(obj$lambda) < 4) { 
-    message(paste0( round(obj$lambda,3) ,"\n"))
+    cat(paste0( round(obj$lambda,3) ,"\n"))
   } else {
-    message(paste0( round(obj$lambda[1:3],3) ,"\n"))
+    cat(paste0( round(obj$lambda[1:3],3) ,"\n"))
   }
 }
 
